from charms.reactive import when, when_not, set_state, remove_state
from charmhelpers.core.host import write_file
from charmhelpers.core import hookenv
from charms.layer.snap import install as snap_install
from subprocess import check_call, check_output, CalledProcessError
from time import sleep
from os import path


def file_to_units(local_path, unit_path):
    """ copy a file from the charm onto our unit(s) """
    if local_path[-3:] == '.py':
        perms = 0o755
    else:
        perms = 0o644

    fh = open(local_path, 'r')
    write_file(
        path=unit_path,
        content=fh.read().encode(),
        owner='root',
        perms=perms,
    )


def snap_connect(connect_from=None, connect_to=None):
    if connect_from and connect_to:
        cmd = ['snap', 'connect']
        cmd.extend([connect_from, connect_to])
        hookenv.log('Connecting {} to {}'.format(connect_from, connect_to))
        check_call(cmd, universal_newlines=True)


def wait_for_path(file_path, timeout=300):
    hookenv.log('Waiting for up to {}s for {}'.format(timeout, file_path))
    seconds_waited = 0
    interval = 5
    while not (path.exists(file_path)):
        hookenv.log('{} does not exist - waiting {}s'.format(file_path, interval))
        sleep(interval)
        seconds_waited += interval
        if seconds_waited >= timeout:
            hookenv.log('Giving up waiting for {}'.format(file_path))
            return
    return


def wait_for_livepatch():
    wait_for_path('/var/snap/canonical-livepatch/current/livepatchd-priv.sock')


def restart_livepatch():
    cmd = ['/usr/sbin/service', 'snap.canonical-livepatch.canonical-livepatchd', 'restart']
    hookenv.log('Restarting canonical-livepatch service')
    check_call(cmd, universal_newlines=True)


@when_not('snap.installed.canonical-livepatch')
def canonical_livepatch_install():
    cmd = ['snap', 'list']
    snap_list = check_output(cmd).decode()
    for line in snap_list.split('\n'):
        if 'canonical-livepatch' in line:
            # snap already installed by someone/something else, remove it
            cmd = ['snap', 'remove', 'canonical-livepatch']
            check_call(cmd, universal_newlines=True)
    kwargs = {'channel': 'beta'}
    snap_install('canonical-livepatch', **kwargs)


@when('snap.installed.canonical-livepatch')
@when_not('canonical-livepatch.connected')
def canonical_livepatch_connect():
    # make our connections
    snap_connect('canonical-livepatch:kernel-module-control', 'ubuntu-core:kernel-module-control')
    snap_connect('canonical-livepatch:hardware-observe', 'ubuntu-core:hardware-observe')
    snap_connect('canonical-livepatch:system-observe', 'ubuntu-core:system-observe')
    snap_connect('canonical-livepatch:network-control', 'ubuntu-core:network-control')

    # restart the system service to pick those up
    restart_livepatch()

    hookenv.status_set('active', 'Ready')
    set_state('canonical-livepatch.connected')


@when('canonical-livepatch.connected')
@when_not('canonical-livepatch.registered')
def update_key():
    # Make sure the service is ready for us before starting
    wait_for_livepatch()

    config = hookenv.config()

    # don't try to set a key when one has already been set, LP#1622782
    previous_key = config.previous('key')
    if previous_key:
        cmd = ['/snap/bin/canonical-livepatch', 'disable']
        hookenv.log('Disabling canonical-livepatch ahead of key change')
        check_call(cmd, universal_newlines=True)

    key = config.get('key')
    if key:
        cmd = ['/snap/bin/canonical-livepatch', 'enable', '{}'.format(key)]
        hookenv.log('Activating canonical-livepatch with your key')
        check_call(cmd, universal_newlines=True)
        set_state('canonical-livepatch.registered')
    else:
        hookenv.log('Unable to activate canonical-livepatch as no key has been set')


@when('config.changed.key')
def unregister_key():
    remove_state('canonical-livepatch.registered')


@when('nrpe-external-master.available')
def update_nrpe_config(nagios):
    config = hookenv.config()
    nagios_hostname = hookenv.local_unit()
    nagios_context = config.get('nagios-context')

    # install nagios support files
    file_to_units(
        'files/check_canonical-livepatch.cron',
        '/etc/cron.d/check_canonical-livepatch',
    )
    file_to_units(
        'files/check_canonical-livepatch.py',
        '/usr/lib/nagios/plugins/check_canonical-livepatch.py',
    )
    file_to_units(
        'files/canonical-livepatch-status.txt',
        '/var/lib/nagios/canonical-livepatch-status.txt',
    )

    nagios.add_check(["/usr/lib/nagios/plugins/check_canonical-livepatch.py", ],
                     name="check_canonical-livepatch",
                     description="Verify canonical-livepatch is working",
                     context=nagios_context,
                     unit=nagios_hostname)
